#include <iostream>
#include <vector>
#include <queue>

#define endl '\n'

using namespace std;

const int MAXN = 5e3 + 3;
const int INF = 1e9;

vector<int> graph[MAXN];
int dist[MAXN];
pair<int, int> min_dist[MAXN][2];

void calculate_distance(int startv, int n)
{
    for (int i = 0; i < n; ++i)
    {
        dist[i] = -1;
    }
    dist[startv] = 0;

    queue<int> bfs;
    bfs.push(startv);

    while (!bfs.empty())
    {
        int currv = bfs.front();
        bfs.pop();

        for (int nextv : graph[currv])
        {
            if (dist[nextv] == -1)
            {
                dist[nextv] = dist[currv] + 1;
                bfs.push(nextv);
            }
        }
    }
}

void calculate_min_distance(int startv, int n)
{
    min_dist[startv][0] = min_dist[startv][1] = {0, startv};

    for (int currv = 0; currv < n; ++currv)
    {
        if (currv == startv)
        {
            continue;
        }

        min_dist[currv][0] = min_dist[currv][1] = {INF, -1};

        for (int nextv : graph[currv])
        {
            if (nextv == startv or dist[nextv] == -1)
            {
                continue;
            }

            if (dist[nextv] < min_dist[currv][0].first)
            {
                min_dist[currv][1] = min_dist[currv][0];
                min_dist[currv][0] = {dist[nextv], nextv};
            }
            else if (dist[nextv] == min_dist[currv][0].first or dist[nextv] < min_dist[currv][1].first)
            {
                min_dist[currv][1] = {dist[nextv], nextv};
            }
        }
    }
}

int find_quadruple(int startv, int n)
{
    calculate_distance(startv, n);
    calculate_min_distance(startv, n);

    int result = INF;
    for (int neighbor1 : graph[startv])
    {
        for (int neighbor2 : graph[neighbor1])
        {
            if (neighbor2 == startv)
            {
                continue;
            }

            if (min_dist[neighbor2][0].first != INF and min_dist[neighbor2][0].second != neighbor1)
            {
                result = min(result, 1 + min_dist[neighbor2][0].first);
            }
            else if (min_dist[neighbor2][1].first != INF)
            {
                result = min(result, 1 + min_dist[neighbor2][1].first);
            }
        }
    }

    return result;
}

int main()
{
    ios::sync_with_stdio(false);
    cin.tie(nullptr);

    int n;
    cin >> n;

    for (int currv = 0; currv < n - 1; ++currv)
    {
        string input;
        cin >> input;

        for (int nextv = currv + 1; nextv < n; ++nextv)
        {
            if (input[nextv - currv - 1] == '1')
            {
                graph[currv].push_back(nextv);
                graph[nextv].push_back(currv);
            }
        }
    }

    int result = INF;
    for (int i = 0; i < n; ++i)
    {
        result = min(result, find_quadruple(i, n));
    }

    cout << (result == INF ? -1 : result) << endl;

    return 0;
}
