#include <algorithm>
#include <iostream>
#include <numeric>
#include <vector>
#include <bitset>

typedef long long llong;
const int MAXN = 5000 + 10;
const int INF  = 1e9;

const int BUFF_SIZE = 1e5;
int buffPos = BUFF_SIZE - 1;
char buff[BUFF_SIZE];

inline void readChar()
{
    if (++buffPos == BUFF_SIZE) fread(buff, BUFF_SIZE, 1, stdin), buffPos = 0;
}

inline void readString(char s[])
{
    int pos = 0;
    for (; buff[buffPos] < '0' || buff[buffPos] > '1'; readChar());
    for (; buff[buffPos] >= '0' && buff[buffPos] <= '1'; readChar())
        s[pos++] = buff[buffPos];
}

inline void readInt(int &num)
{
    num = 0;
    for (; buff[buffPos] < '0' || buff[buffPos] > '9'; readChar());
    for (; buff[buffPos] >= '0' && buff[buffPos] <= '9'; readChar())
        num = 10 * num + buff[buffPos] - '0';
}


struct Edge
{
    int to;
    int prev;
};

int n;
int cntE;
int g[MAXN];
int ptr[MAXN];
int perm[MAXN];
bool bs[MAXN][MAXN];
int common[MAXN][MAXN];
Edge edges[MAXN * MAXN];
std::bitset <MAXN> pairs[MAXN];
std::bitset <MAXN> neigh;
std::bitset <MAXN> vis;

inline void addEdge(int from, int to)
{
    edges[cntE] = {to, ptr[from]};
    ptr[from] = cntE++;
}

void solve()
{
    int j;
    for (int i = 1 ; i <= n ; ++i)
    {
        int k;
        vis.reset();
        for (int j1 = ptr[i] ; j1 != -1 ; j1 = edges[j1].prev)
        {
            j = edges[j1].to;
            for (int k1 = ptr[j] ; k1 != -1 ; k1 = edges[k1].prev)
            {
                k = edges[k1].to;
                if (i == k) continue;
                if (vis[k])
                {
                    std::cout << 2 << '\n';
                    return;
                }

                vis[k] = 1;
                common[i][k] = j;
                pairs[i][k] = 1;
            }
        }
    }

    for (int i = 1 ; i <= n ; ++i)
    {
        for (int k = 1 ; k <= n ; ++k)
        {
            if (pairs[i][k] && bs[i][k] && g[common[i][k]] > 2)
            {
                std::cout << 3 << '\n';
                return;
            }
        }
    }

    for (int i = 1 ; i <= n ; ++i)
    {
        for (int k = i + 1 ; k <= n ; ++k)
        {
            if (!pairs[i][k] || bs[i][k]) continue;
            for (int j1 = ptr[k] ; j1 != -1 ; j1 = edges[j1].prev)
            {
                j = edges[j1].to;
                if (common[i][j])
                {
                    std::cout << 3 << '\n';
                    return;
                }
            }
        }
    }

    for (int i = 1 ; i <= n ; ++i)
    {
        if (g[i] == 1) continue;
        for (int j1 = ptr[i] ; j1 != -1 ; j1 = edges[j1].prev)
        {
            j = edges[j1].to;
            if (g[i] >= 3 && g[j] >= 2)
            {
                std::cout << 4 << '\n';
                return;
            }

            if (g[i] == 2 && g[j] == 2)
            {
                int curr = edges[ptr[i]].to;
                if (curr == j) curr = edges[edges[ptr[i]].prev].to;
                int curr2 = edges[ptr[j]].to;
                if (curr2 == i) curr2 = edges[edges[ptr[j]].prev].to;
                if (curr != curr2)
                {
                    std::cout << 4 << '\n';
                    return;
                }
            }
        }
    }

    std::cout << -1 << '\n';
}

char s[MAXN];
void input()
{
    readInt(n);
    std::fill(ptr + 1, ptr + 1 + n, -1);
    for (int i = 1 ; i < n ; ++i)
    {
        readString(s);
        for (int j = i + 1 ; j <= n ; ++j)
        {
            bs[i][j] = s[j - i - 1] - '0';
            bs[j][i] = s[j - i - 1] - '0';
            if (s[j - i - 1] == '1')
            {
                g[i]++; g[j]++;
                addEdge(i, j);
                addEdge(j, i);
            }
        }
    }
}

void fastIO()
{
    std::ios_base :: sync_with_stdio(0);
    std::cout.tie(nullptr);
    std::cin.tie(nullptr);
}

int main()
{
    srand(229295);
    fastIO();
    input();
    solve();

    return 0;
}
